@@ -1,5 +1,5 @@
 #-------------------------------------------------------------------------
-# The Azure Batch Apps Python Client ver. 0.1.0
+# The Azure Batch Apps Python Client
 #
 # Copyright (c) Microsoft Corporation. All rights reserved. 
 #
@@ -41,6 +41,8 @@ from .exceptions import (
 import logging
 
 CRED_STORE = 'AzureBatchApps'
+VERIFY = True
+CA_CERT = None
 
 
 def _http(base_uri, *extra):
@@ -82,36 +84,40 @@ class AzureOAuth(object):
     session.
     """
 
-    session = None
     config = None
+    state = None
     LOG = logging.getLogger('batch_apps')
 
     @staticmethod
-    def _setup_session(auth, client, state=None):
+    def _setup_session(auth):
         """Apply configuration info to set up a new OAuth2 session."""
 
-        AzureOAuth.LOG.debug("Configuring session. Client: {0}, Cfg: "
-                  "{1}".format(str(client), str(auth)))
+        AzureOAuth.LOG.debug("Configuring session with {0}".format(str(auth)))
+            
+        redirect = _http(auth.get('redirect_uri'))
 
-        if (not utils.valid_keys(client, ['client_id', 'redirect_uri']) or
-            not 'token_uri' in auth):
+        new_session = requests_oauthlib.OAuth2Session(
+                auth.get('client_id'),
+                redirect_uri=redirect,
+                state=AzureOAuth.state)
 
-            raise InvalidConfigException(
-                "Correct authentication configuration not found")
+        new_session.verify = VERIFY
+        if CA_CERT and VERIFY is True:
+            new_session.verify = CA_CERT
+        return new_session
 
-        if not AzureOAuth.session:
-            redirect = _http(client.get('redirect_uri'))
+    @staticmethod
+    def _check_state(response, state):
+        state_key = '&state='
+        state_idx = response.find(state_key)
+        if state_idx < 0:
+            return False
 
-            if isinstance(state, str):
-                AzureOAuth.session = requests_oauthlib.OAuth2Session(
-                    client.get('client_id'),
-                    redirect_uri=redirect,
-                    state=state)
+        strt_idx = state_idx + len(state_key)
+        end_idx = response.find('&', strt_idx)
+        state_val = response[strt_idx:end_idx]
 
-            else:
-                AzureOAuth.session = requests_oauthlib.OAuth2Session(
-                    client.get('client_id'),
-                    redirect_uri=redirect)
+        return state_val == state
 
     @staticmethod
     def get_session(config=None):
@@ -129,15 +135,16 @@ class AzureOAuth(object):
             - A :class:`.InvalidConfigException` if the supplied,
               or default configuration does not contain the necessary
               authentication data.
-            - A :class:`.AuthenticationException` if there's not existing
+            - A :class:`.AuthenticationException` if there's no existing
               stored session or the token is invalid.
         """
         AzureOAuth.config = config if config else Configuration()
-        return Credentials(AzureOAuth.config,
-                           AzureOAuth.config.get('client_id'))
+        auth = AzureOAuth.config.aad_config()
+        return Credentials(AzureOAuth.config, auth['client_id'])
 
     @staticmethod
-    def get_authorization_url(config=None, prompt=False, **additional_args):
+    def get_authorization_url(config=None, msa=False, prompt=False,
+                              state=None, **additional_args):
         """
         Construct client-specific authentication URL. This URL can be used
         in a web browser to direct the user to log in and authenticate
@@ -147,8 +154,11 @@ class AzureOAuth(object):
             - config (:class:`.Configuration`): A custom configuration object.
               Default is `None` where a default :class:`.Configuration` will
               be created.
+            - msa (bool): Authenticate by MSA. Default is ``False``.
             - prompt (bool): Force login prompt regardless of existing session.
               Default is ``False``.
+            - state (str): A guid for validating the state of the server
+              communication.
             - additional_args (dict): Any additional AAD parameters to include
               in the URL
 
@@ -171,19 +181,28 @@ class AzureOAuth(object):
             raise InvalidConfigException(
                 "Correct authentication configuration not found")
 
-        auth = AzureOAuth.config.aad_config()
-        AzureOAuth._setup_session(auth, AzureOAuth.config.default_params())
-
-        #if msa:
-        #    additional_args['domain_hint'] = 'live.com'
+        if msa:
+            additional_args['domain_hint'] = 'live.com'
 
         if prompt:
             additional_args['prompt'] = 'login'
 
+        if state:
+            AzureOAuth.state = state
+        else:
+            AzureOAuth.state = requests_oauthlib.oauth2_session.generate_token()
+
+        auth = AzureOAuth.config.aad_config()
+        session = AzureOAuth._setup_session(auth)
+
         try:
-            auth_url, state = AzureOAuth.session.authorization_url(
-                _https(auth['auth_uri']),
-                resource=_https(auth['resource']),
+            auth_uri = _https(auth['root'], auth['tenant'], auth['auth_uri'])
+            AzureOAuth.LOG.debug("Generating URL with {0}, {1}, {2}".format(
+                auth_uri, _https(auth['resource']), additional_args))
+
+            auth_url, state = session.authorization_url(
+                auth_uri,
+                resource=_https(auth['resource']), #DEP
                 **additional_args)
 
             return auth_url, state
@@ -204,7 +223,9 @@ class AzureOAuth(object):
             - config (:class:`.Configuration`): A custom configuration object.
               Default is `None` where a default :class:`.Configuration` will
               be created.
-            - state (str): A state guid for auth server validation.
+            - state (str): A state guid for auth server validation. This is not
+              necessary if :meth:`.get_authorization_url` was used to generate
+              the url, as state will already be set. If set, will override.
 
         :Returns:
             - An authenticated :class:`.Credentials` object.
@@ -224,24 +245,28 @@ class AzureOAuth(object):
                 "Correct authentication configuration not found")
 
         auth = AzureOAuth.config.aad_config()
-        AzureOAuth._setup_session(auth,
-                                  AzureOAuth.config.default_params(),
-                                  state=state)
+        session = AzureOAuth._setup_session(auth)
 
-        redirect = AzureOAuth.config.get('redirect_uri')
+        state = state if state else AzureOAuth.state
+        valid_state = AzureOAuth._check_state(auth_url, state)
+        if not valid_state:
+            raise AuthenticationException(
+                "Security error: State not equal in request and response.")
+
+        redirect = auth.get('redirect_uri')
 
         if auth_url.startswith(_http(redirect)):
-            auth_url = auth_url.replace('http', 'https')
+            auth_url = _https(auth_url)
 
         elif not auth_url.startswith(_https(redirect)):
             auth_url = _https(redirect) + auth_url
 
+        token_uri = _https(auth['root'], auth['tenant'], auth['token_uri'])
         try:
             AzureOAuth.LOG.debug("Fetching token with token_uri: "
-                                 "{0}".format(_https(auth['token_uri'])))
+                                 "{0}".format(token_uri))
 
-            token = AzureOAuth.session.fetch_token(
-                _https(auth['token_uri']),
+            token = session.fetch_token(token_uri,
                 authorization_response=auth_url)
 
         except oauth2.rfc6749.errors.InvalidGrantError as excp:
@@ -253,14 +278,37 @@ class AzureOAuth(object):
         except oauth2.rfc6749.errors.MismatchingStateError as excep:
             raise AuthenticationException(excp.description)
 
-        authorized_creds = Credentials(AzureOAuth.config,
-                                       AzureOAuth.config.get('client_id'),
-                                       token=token)
+        authorized_creds = Credentials(AzureOAuth.config, auth['client_id'], token=token)
         return authorized_creds
 
     @staticmethod
     def get_principal_token(config=None):
-        """Retrieve a Service Principal access token from AAD server.
+        """
+        .. warning:: Deprecated. Please use :meth:`.get_unattended_session()`.
+        Retrieve a Unattended Account access token from AAD server.
+
+        :Kwargs:
+            - config (:class:`.Configuration`): A custom configuration object.
+              Default is `None` where a default :class:`.Configuration`
+              will be created.
+
+        :Returns:
+            - An authenticated :class:`.Credentials` object.
+
+        :Raises:
+            - A :class:`.InvalidConfigException` if the supplied, or default
+              configuration does not contain the necessary authentication
+              data.
+            - A :class:`.AuthenticationException` if the supplied credentials
+              are invalid (e.g. have expired).
+        """
+        AzureOAuth.LOG.warning("get_principal_token() is deprecated. "
+                               "Please use get_unattended_session().")
+        return AzureOAuth.get_unattended_session(config=config)
+
+    @staticmethod
+    def get_unattended_session(config=None):
+        """Retrieve a Unattended Account access token from AAD server.
 
         :Kwargs:
             - config (:class:`.Configuration`): A custom configuration object.
@@ -284,37 +332,38 @@ class AzureOAuth(object):
             raise InvalidConfigException(
                 "Correct authentication configuration not found")
 
-        auth = AzureOAuth.config.aad_config()
-
+        auth = AzureOAuth.config.aad_config(unattended=True)
         try:
-            secret = auth['service_principal_key']
-            service = auth['service_principal'].split(';')
-            client_id = service[0].split('=')[1]
-            tenant = service[1].split('=')[1]
-            token_uri = auth['token_uri'].replace("common", tenant)
+            account_id = auth['unattended_account'].split(';')
+            client_id = account_id[0].split('=')[1]
+            tenant = account_id[1].split('=')[1]
 
-        except KeyError:
-            raise InvalidConfigException(
-                "Supplied config does not contain Service Principal auth.")
+        except IndexError as exp:
+            raise InvalidConfigException("Unattended Account must be in the "
+                                         "format 'ClientID=abc;TenantID=xyz'")
 
-        except (AttributeError, IndexError):
-            raise InvalidConfigException(
-                "service_principal must be in the format ClientId=abc;TenantId=abc")
+        secret = auth['unattended_key']
+        token_uri = _https(auth['root'], tenant, auth['token_uri'])
 
         silent_session = requests_oauthlib.OAuth2Session(
             client_id,
             client=BackendApplicationClient(client_id))
+        
+        verification = VERIFY
+        if CA_CERT and VERIFY is True:
+            verification = CA_CERT
 
         try:
             AzureOAuth.LOG.debug("Fetching token with token_uri: "
-                                 "{0}".format(_https(token_uri)))
+                                 "{0}".format(token_uri))
 
             token = silent_session.fetch_token(
-                _https(token_uri),
+                token_uri,
                 client_id=client_id,
-                resource=_https(auth['resource']),
+                resource=_https(auth['resource']), #DEP
                 client_secret=secret,
-                response_type="client_credentials")
+                response_type="client_credentials",
+                verify=verification)
 
         except oauth2.rfc6749.errors.InvalidGrantError as excp:
             raise AuthenticationException(excp.description)
@@ -322,8 +371,7 @@ class AzureOAuth(object):
         except oauth2.rfc6749.errors.OAuth2Error as excp:
             raise AuthenticationException(excp.description)
 
-        authorized_creds = Credentials(AzureOAuth.config,
-                                       client_id,
+        authorized_creds = Credentials(AzureOAuth.config, client_id,
                                        token=token)
         return authorized_creds
 
@@ -343,8 +391,7 @@ class Credentials(object):
         :Args:
             - config (:class:`.Configuration`): A configuration object to
               define the client session.
-            - client_id (str): The client ID for the application as defined
-              in the Configuration.
+            - client_id (str): The client ID as defined in the Configuration.
 
         :Kwargs:
             - token (dict): An authentication token, if not provided will try
@@ -357,20 +404,23 @@ class Credentials(object):
         """
 
         self._log = logging.getLogger('batch_apps')
+        self._id = str(client_id)
 
         if not hasattr(config, "aad_config"):
             raise InvalidConfigException(
                 "Correct authentication configuration not found")
 
-        self.cfg = config.aad_config()
-        self._id = client_id
-
         if not token:
             self._log.debug("No token supplied, attempting to "
                             "retrieve previous session.")
 
+            self.cfg = config.aad_config(unattended=False)
             token = self.get_stored_auth()
 
+        else:
+            unattended = 'refresh_token' not in token
+            self.cfg = config.aad_config(unattended=unattended)
+
         self.token = token
 
         if not utils.valid_keys(self.token, ['token_type', 'access_token']):
@@ -379,7 +429,6 @@ class Credentials(object):
         self.store_auth(self.token)
         self.get_session()
 
-
     def get_session(self):
         """
         Generate the authenticated OAuth2 session.
@@ -391,8 +440,7 @@ class Credentials(object):
             - :class:`.AuthenticationException` if the token is invalid
               or expired.
         """
-        resource = _https(self.cfg['resource'])
-        refresh = _https(self.cfg['token_uri'])
+        resource = _https(self.cfg['resource']) #DEP
         countdown = float(self.token['expires_at']) - time.time()
 
         self.token['expires_in'] = countdown
@@ -400,7 +448,11 @@ class Credentials(object):
 
         try:
             if 'refresh_token' in self.token:
-                return requests_oauthlib.OAuth2Session(
+
+                refresh = _https(self.cfg['root'], self.cfg['tenant'],
+                         self.cfg['token_uri'])
+
+                new_session = requests_oauthlib.OAuth2Session(
                     self._id,
                     token=self.token,
                     auto_refresh_url=refresh,
@@ -409,7 +461,7 @@ class Credentials(object):
                     token_updater=self.store_auth)
 
             else:
-                return requests_oauthlib.OAuth2Session(self._id,
+                new_session = requests_oauthlib.OAuth2Session(self._id,
                                                        token=self.token)
 
         except oauth2.rfc6749.errors.TokenExpiredError as excp:
@@ -419,8 +471,14 @@ class Credentials(object):
             raise AuthenticationException(
                 "Token expired: {0}".format(excp.description))
 
+        else:
+            new_session.verify = VERIFY
+            if CA_CERT and VERIFY is True:
+                new_session.verify = CA_CERT
+            return new_session
+
     def get_stored_auth(self):
-        """Retrieve a previous stored access token for refreshing
+        """Retrieve a previously stored access token for refreshing
 
         :Returns:
             - The token as a dictionary.
